// @ myth - client - cover.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "client/cover.h"
#include "common/error.h"

// @@ const

// Number of reserved lines at top of screen.
static const int CoverTop = 2;

// @@ class COVER

// @@@ void COVER::display()

void COVER::display()
{
    if(page.remain() < 1)
    {
	if(retry)
	    page.reset();

	if(!empty)
	{
	    cover();
	    empty = TRUE;
	}
	return;
    }
    coor used(0, min(page.remain(), size().y - CoverTop));
    
    for(int y = 0; y < used.y; y++)
    {
	if(page.current().length() >= used.x)
	    used.x = page.current().length();

	page.next();
    }
    page.reset(page.where() - used.y);

    PICTURE picture(used);
    
    for(coor pos = Origo; pos.y < used.y; pos.y++)
    {
	if(page.string() != NULL)
	    for(pos.x = 0; page.string()[pos.x] != 0; pos.x++)
		picture[pos] = page.string()[pos.x];

	page.next();
    }
    cover(picture, coor(max(0, size().x - used.x), CoverTop));
    empty = FALSE;
}    

// @@@ void COVER::operator()()

void COVER::operator()()
{
    display();
}

// @@@ void COVER::operator()(PAGE* text, bool again = FALSE)

void COVER::operator()(PAGE* text, bool again = FALSE)
{
    retry = again;
    page.clear();
    while(text->remain() > 0)
    {
	page.append(text->current());
	text->next();
    }
    display();
}

// @@@ void COVER::clear()

void COVER::clear()
{
    page.clear();
    retry = FALSE;
}

// @@@ COVER::COVER()

COVER::COVER() : (WINDOW_NONE)
{
    empty = TRUE;
    retry = FALSE;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
