// @ myth - client - dungeon.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "client/dungeon.h"
#include "common/error.h"
#include "client/myth.h"
#include "client/global.h"

// @@ const

// Time to show a speciel effect on the screen.
static const EffectTime = 200000;

// @@ class DUNGEON

// @@@ void DUNGEON::update()

void DUNGEON::update()
{
    char* string = new char[size().x + 1];

    for(int y = 0; y < size().y; y++)
    {
	if(touched[y] == FALSE)
	    continue;
	
	int x;
	// Skip trailing spaces.
	for(x = size().x; x > 0; x--)
	    if(map.what(coor(x - 1, y) + corner) != ' ')
		break;
	string[x] = 0;
	// Copy to start of line.
	while(x > 0)
	{
	    x--;
	    string[x] = map.what(coor(x, y) + corner);
	}
	WINDOW::print(coor(0, y), string);
	touched[y] = FALSE;
    }
    delete string;
}

// @@@ * void DUNGEON::clear()

void DUNGEON::clear()
{
    map.clear();
    for(int i = 0; i < size().y; i++)
	touched[i] = TRUE;
}

// @@@ void DUNGEON::load(BINARY* binary)

void DUNGEON::load(BINARY* binary)
{ map.load(binary); }

// @@@ * void DUNGEON::plot(coor pos, char c)

void DUNGEON::plot(coor pos, char c)
{
    coor loc = pos - corner;

    if(OnWindow(loc))
	touched[loc.y] = TRUE;

    map.plot(pos, c);
}

// @@@ * void DUNGEON::print(coor pos, STRING string)

void DUNGEON::print(coor pos, STRING string)
{
    if(string.length() == 0)
	return;

    coor loc = pos - corner;

    if(OnWindow(loc))
	touched[loc.y] = TRUE;

    map.print(pos, string);
}

// @@@ * void DUNGEON::effect(coor pos, char c)

void DUNGEON::effect(coor pos, char c)
{
    coor location = pos - corner;

    if(OnWindow(location))
    {
	update();
	WINDOW::plot(location, c);
	WINDOW::cursor(location);
	GlobalTTY->flush();
	(void) usleep(EffectTime);
	WINDOW::plot(location, map.what(pos));
    }
}

// @@@ * void DUNGEON::flush(coor pos)

void DUNGEON::flush(coor pos)
{
    coor location = position + pos - corner;
    coor margin(global.border(), global.border());

    if(!OnWindow(location + margin) || !OnWindow(location - margin))
    {
	center(pos);
	location = position + pos - corner;
    }
    update();
    cursor(location);
}

// @@@ void DUNGEON::center(coor pos)

void DUNGEON::center(coor pos)
{
    coor target = position + pos - size() / 2;

    if(target != corner)
    {
	corner = target;
	for(int y = 0; y < size().y; y++)
	    touched[y] = TRUE;
    }
}

// @@@ DUNGEON::DUNGEON()

DUNGEON::DUNGEON() : (WINDOW_ALL)
{
    position = Origo;
    corner = Origo;
    touched = new bool[size().y];
    for(int i = 0; i < size().y; i++)
	touched[i] = FALSE;
}

// @@@ DUNGEON::~DUNGEON()

DUNGEON::~DUNGEON()
{
    delete touched;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
