// @ myth - client - window.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef WINDOW_HEADER
#define WINDOW_HEADER

// @@ include

#include "client/screen.h"
#include "common/string.h"

// @@ extern

extern void SelectScreen(SCREEN* screen);

// @@ class WINDOW

// @@@ enum edge

enum edge { WINDOW_ALL, WINDOW_LEFT, WINDOW_RIGHT, WINDOW_TOP,
	    WINDOW_BOTTOM, WINDOW_NONE };

// @@@ class WINDOW

class WINDOW
{
    SCREEN* screen;
    coor corner;
    coor volume;
public:
    inline coor ToScreen(coor pos)
    { return pos + corner; }
    inline coor size()
    { return volume; }
    inline bool OnWindow(coor pos)
    { return pos >= Origo && pos < size(); }
    inline void cursor(coor pos)
    { if(OnWindow(pos)) screen->cursor(ToScreen(pos)); }
    inline void plot(coor pos, char c)
    { if(OnWindow(pos)) screen->plot(ToScreen(pos), c); }
    void print(coor pos, const char* string);
    void print(coor pos, STRING string);
    inline char what(coor pos)
    { return screen->what(ToScreen(pos)); }
    inline void redraw()
    { screen->redraw(); }
    inline void cover()
    { screen->cover(); }
    inline void cover(const PICTURE& window, coor pos)
    { screen->cover(window, pos); }
    WINDOW(edge where, int size = 1, int percent = 0, int minimum = 0);
};

// @@ end

#endif WINDOW_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
