// @ myth - common - map.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef MAP_HEADER
#define MAP_HEADER

#include "common/common.h"

// @@ include

#include "common/binary.h"

// @@ const

// Extra space allocated, when the map is extended.
static const int ExtraMap = 5;

// @@ (local) class map_line

class map_line
{
    char* data;			// Pointer to data
    int first;			// First column mapped
    int last;			// Last column mapped
    // Send content of data to a buffer
    void send(int start, int line, int stop, BINARY* binary);
public:
    void assign(int x, int length, const char* buffer, char EmptyMap);
    inline char what(int x, char EmptyMap)
    { return (data == NULL || x < first || x > last)
	  ? EmptyMap : data[x - first]; }
    const map_line& operator=(const map_line& line);
    void save(BINARY* binary, int line, char EmptyMap);
    map_line();
    ~map_line();
};

// @@ class MAP

class MAP
{
    char EmptyMap;		// Character returned when unmapped.
    int first;			// First line mapped.
    int last;			// Last line mapped.
    map_line* data;		// Array of lines.
    void resize(int from, int to);
    void assign(coor pos, int length, const char* buffer);
public:
    // * Draw the map
    void clear();
    inline void plot(coor pos, char c)
    { assign(pos, 1, &c); }
    inline void print(coor pos, STRING string)
    { assign(pos, string.length(), string()); }
    inline char what(coor pos)
    { return (pos.y < first || pos.y > last)
	  ? EmptyMap : data[pos.y - first].what(pos.x, EmptyMap); }
    // * Communication
    void save(BINARY* binary);
    void load(BINARY* binary);
    void SaveDefault(BINARY* binary);
    void LoadDefault(BINARY* binary);
    // * Misc
    void SelectDefault(char DefaultMap)
    { EmptyMap = DefaultMap; }
    MAP();
    ~MAP();
};

// @@ end

#endif MAP_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
