// @ myth - server - party.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef PARTY_HEADER
#define PARTY_HEADER

// @@ include

#include "common/list.h"
#include "common/config.h"
#include "common/string.h"

struct PAGE;
struct PARSE;
struct OBJECT;
struct MONSTER;
struct PLAYER;
struct BINARY;

// @@ OBJECT_LIST

class OBJECT_LIST
{
    LIST list;
public:
    STRING OBJECT_LIST::PackPage(char symbol, PAGE* FullPage, PLAYER* player);
    void add(OBJECT* object)
    { list.add(object); }
    void insert(OBJECT* object)
    { list.insert(object); }
    void append(OBJECT* object)
    { list.append(object); }
    void remove()
    { list.remove(); }
    void reset(int target = 0)
    { list.reset(target); }
    void next()
    { list.next(); }
    inline OBJECT* current()
    { return (OBJECT*) list.current(); }
    void erase();
    void CreatePage(PAGE* page, PLAYER* player = NULL);
    void sort();
    void clear()
    { list.clear(); }
    int length()
    { return list.length(); }
    int remain()
    { return list.remain(); }
    int where()
    { return list.where(); }
    void load(PARSE* parse, coor pos);
    void save(BINARY* binary);
    void load(BINARY* binary);
    OBJECT_LIST();
    OBJECT_LIST(PARSE* parse, coor pos);
};

// @@ MONSTER_LIST

class MONSTER_LIST : public OBJECT_LIST
{
public:
    void add(MONSTER* monster)
    { OBJECT_LIST::add((OBJECT*) monster); }
    void insert(MONSTER* monster)
    { OBJECT_LIST::insert((OBJECT*) monster); }
    void append(MONSTER* monster)
    { OBJECT_LIST::append((OBJECT*) monster); }
    inline MONSTER* current()
    { return (MONSTER*) OBJECT_LIST::current(); }
};

// @@ PLAYER_LIST

class PLAYER_LIST : public MONSTER_LIST
{
public:
    void add(PLAYER* player)
    { MONSTER_LIST::add((MONSTER*) player); }
    void insert(PLAYER* player)
    { MONSTER_LIST::insert((MONSTER*) player); }
    void append(PLAYER* player)
    { MONSTER_LIST::append((MONSTER*) player); }
    inline PLAYER* current()
    { return (PLAYER*) MONSTER_LIST::current(); }
};

// @@ end

#endif PARTY_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
