// @ myth - server - scroll.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "server/scroll.h"
#include "server/level.h"
#include "server/ability.h"
#include "common/error.h"
#include "server/rnd.h"
#include "server/question.h"
#include "server/player.h"
#include "common/format.h"

struct CHANNEL;

// @@ SCROLL_KNOWLEDGE

// @@@ cont SCROLL_KNOWLEDGE::CheckIt(MONSTER* monster)

cont SCROLL_KNOWLEDGE::CheckIt(MONSTER* monster)
{
    monster->learn(this);

    if(monster->IsConfused())
	monster->message("You identify this as an identify scroll.");
    else if(IsCursed())
    {
	monster->message("Hi, I don't remember seeing Maud in this game...");
	monster->forget(MaxValue + 1 - MyValue);
    }
    else if(IsBlessed())
    {
	monster->message("You feel wise.");
	
	SPECIFY_ALL specify;
	OBJECT_LIST list;

	monster->pack.RemoveHalf(&specify, &list);

	list.reset();
	while(list.current() != NULL)
	{
	    monster->identify(list.current());
	    list.next();
	}
	monster->pack.add(&list);
    }
    else
	return CONTINUE;

    return DONE;
}

// @@@ void SCROLL_KNOWLEDGE::effect(LEVEL*, MONSTER* monster)

void SCROLL_KNOWLEDGE::effect(LEVEL*, MONSTER* monster)
{
    if(CheckIt(monster) == DONE)
	/* Do nothing */;
    else
    {
	symbol_type symbol = ALL_SYMBOLS[rnd(0, SymbolCount - 1)];
	
	SPECIFY_SYMBOL specify(symbol);
	OBJECT_LIST list;

	monster->pack.RemoveHalf(&specify, &list);

	if(list.length() > 0)
	{
	    monster->message(format("You identify all your %s.",
				    SymbolName(symbol, PLURAL)));
	    
	    list.reset();
	    while(list.current() != NULL)
	    {
		monster->identify(list.current());
		list.next();
	    }
	    monster->pack.add(&list);
	}
	else
	    monster->message(STRING("The spell failed...", STRING_STATIC));
    }
}

// @@@ QUESTION* SCROLL_KNOWLEDGE::DoEffect(LEVEL*,

QUESTION* SCROLL_KNOWLEDGE::DoEffect(LEVEL*,
				   PLAYER* player, CHANNEL* channel)
{
    if(CheckIt(player) == CONTINUE)
	return new QUESTION_IDENTIFY(MyValue, channel);
    else
	return NULL;
}

// @@ SCROLL_PRAYERS

// @@@ cont SCROLL_PRAYERS::CheckIt(MONSTER* monster)

cont SCROLL_PRAYERS::CheckIt(MONSTER* monster)
{
    monster->learn(this);

    if(!rnd.chance(MyValue, MaxValue))
	monster->message("The gods ignore you.");
    else if(monster->IsConfused())
    {
	monster->message(format("Your pack glows %s for a moment.",
				IsCursed() ? "orange" : "purple"));

	SPECIFY_WORN specify(monster);
	OBJECT_LIST list;
	
	monster->pack.RemoveHalf(&specify, &list);

	list.reset();
	while(list.current() != NULL)
	{
	    if(IsCursed())
		list.current()->curse();
	    else if(IsBlessed())
		list.current()->bless();
	    else
		list.current()->RemoveCurse();

	    list.next();
	}
	monster->pack.add(&list);
    }
    else
	return CONTINUE;

    return DONE;
}

// @@@ void SCROLL_PRAYERS::pray(MONSTER* monster)

void SCROLL_PRAYERS::pray(MONSTER* monster)
{
    monster->message(format("Your pack glows %s for a moment.",
			    IsCursed() ? "red" : "blue"));

    SPECIFY_ALL specify;
    OBJECT_LIST list;

    monster->pack.RemoveHalf(&specify, &list);

    list.reset();
    while(list.current() != NULL)
    {
	if(IsCursed())
	    list.current()->curse();
	else
	    list.current()->RemoveCurse();

	list.next();
    }
    monster->pack.add(&list);
}

// @@@ void SCROLL_PRAYERS::effect(LEVEL*, MONSTER* monster)

void SCROLL_PRAYERS::effect(LEVEL*, MONSTER* monster)
{
    if(CheckIt(monster) == CONTINUE)
	pray(monster);
}

// @@@ QUESTION* SCROLL_PRAYERS::DoEffect(LEVEL*,

QUESTION* SCROLL_PRAYERS::DoEffect(LEVEL*, PLAYER* player, CHANNEL* channel)
{
    if(CheckIt(player) == DONE)
	/* Do nothing */;
    else if(IsBlessed())
	return new QUESTION_BLESS(BLESSED, channel);
    else if(IsCursed() && MyValue < MaxValue / 2)
	return new QUESTION_BLESS(CURSED, channel);
    else if(MyValue < MaxValue / 3)
	return new QUESTION_BLESS(NORMAL, channel);
    else
	pray(player);

    return NULL;
}

// @@ SCROLL_MAGIC

// @@@ cont SCROLL_MAGIC::CheckIt(MONSTER* monster)

cont SCROLL_MAGIC::CheckIt(MONSTER* monster)
{
    monster->learn(this);

    if(monster->IsConfused())
    {
	monster->message(format("Your pack glows %s for a moment.",
				IsCursed() ? "orange" : "purple"));

	SPECIFY_SYMBOL specify(SYMBOL_FOOD);
	OBJECT_LIST list;
	int change;

	if(IsCursed())
	    change = -MyValue / 5 - 1;
	else if(IsBlessed())
	    change = MyValue + 1;
	else
	    change = MyValue / 5 + 1;
	
	monster->pack.RemoveHalf(&specify, &list);

	list.reset();
	while(list.current() != NULL)
	{
	    list.current()->ChangeValue(change);
	    list.next();
	}
	monster->pack.add(&list);
    }
    else if(IsBlessed())
    {
	monster->message(format("Your pack glows %s for a moment.",
				IsCursed() ? "yellow" : "green"));

	SPECIFY_ALL specify;
	OBJECT_LIST list;

	monster->pack.RemoveHalf(&specify, &list);

	list.reset();
	while(list.current() != NULL)
	{
	    list.current()->ChangeValue(MyValue / 5 + 1);
	    list.next();
	}
	monster->pack.add(&list);
    }
    else
	return CONTINUE;

    return DONE;
}

// @@@ void SCROLL_MAGIC::effect(LEVEL*, MONSTER* monster)

void SCROLL_MAGIC::effect(LEVEL*, MONSTER* monster)
{
    if(CheckIt(monster) == CONTINUE)
    {
	monster->message(format("Your pack glows %s for a moment.",
				IsCursed() ? "yellow" : "green"));

	SPECIFY_SYMBOL specify(SYMBOL_WEAPON);
	OBJECT_LIST list;
	int change;

	if(IsCursed())
	    change = -MyValue / 5 - 1;
	else if(IsBlessed())
	    change = MyValue + 1;
	else
	    change = MyValue / 5 + 1;
	
	monster->pack.RemoveHalf(&specify, &list);

	list.reset();
	while(list.current() != NULL)
	{
	    list.current()->ChangeValue(change);
	    list.next();
	}
	monster->pack.add(&list);
    }
}

// @@@ QUESTION* SCROLL_MAGIC::DoEffect(LEVEL*, 

QUESTION* SCROLL_MAGIC::DoEffect(LEVEL*, PLAYER* player, CHANNEL* channel)
{
    if(CheckIt(player) == CONTINUE)
	return new QUESTION_ENCHANT(IsCursed() ? -MyValue : MyValue, channel);

    return NULL;
}

// @@ SCROLL_TELEPORT

// @@@ cont SCROLL_TELEPORT::CheckIt(LEVEL*, MONSTER* monster)

cont SCROLL_TELEPORT::CheckIt(LEVEL*, MONSTER* monster)
{
    if(monster->IsConfused())
    {
	level_type OldDepth = monster->depth();
	
	if(IsCursed())
	    monster->NewLevel(LEVEL_HELL, '<');
	else if(IsBlessed())
	    monster->NewLevel(LEVEL_HOME, '<');
	else
	    monster->NewLevel((level_type) rnd(1, global.DungeonSize() - 1),
			      '<');

	if(monster->depth() != OldDepth)
	{
	    monster->learn(this);

	    if(IsCursed())
		monster->message("You hear an satanical laughter nearby.");
	    else if(IsBlessed())
		monster->message("You feel yourself yanked upwards!");
	    else
		monster->message("The air about you is charged.");
	}
	return DONE;
    }
    return CONTINUE;
}

// @@@ void SCROLL_TELEPORT::teleport(LEVEL* level, MONSTER* monster)

void SCROLL_TELEPORT::teleport(LEVEL* level, MONSTER* monster)
{
    coor OldPosition = monster->position();
	
    monster->teleport(level, (MyValue * 100) / MaxValue, holy);
	
    if(monster->position() != OldPosition)
	monster->learn(this);
}

// @@@ void SCROLL_TELEPORT::effect(LEVEL* level, MONSTER* monster)

void SCROLL_TELEPORT::effect(LEVEL* level, MONSTER* monster)
{
    if(CheckIt(level, monster) == CONTINUE)
	teleport(level, monster);
}

// @@@ QUESTION* SCROLL_TELEPORT::DoEffect(LEVEL* level,

QUESTION* SCROLL_TELEPORT::DoEffect(LEVEL* level,
				  PLAYER* player, CHANNEL* channel)
{
    if(CheckIt(level, player) == DONE)
	/* Do nothing */;
    else if(IsBlessed() || player->CheckAbility(ABILITY_TELEPORT_CONTROL))
    {
	player->learn(this);
	return new QUESTION_TELEPORT(MyValue, channel);
    }
    else
	teleport(level, player);
    
    return NULL;
}

// @@ SCROLL_MAPPING

// @@@ void SCROLL_MAPPING::effect(LEVEL*, MONSTER* monster)

void SCROLL_MAPPING::effect(LEVEL*, MONSTER* monster)
{
    monster->learn(this);
    monster->GainExperience(MyValue);
}

// @@@ QUESTION* SCROLL_MAPPING::DoEffect(LEVEL*, PLAYER* player, CHANNEL*)

QUESTION* SCROLL_MAPPING::DoEffect(LEVEL*, PLAYER* player, CHANNEL*)
{
    player->learn(this);

    if(IsCursed())
	player->CreateMap(MAPPING_FORGET, MyValue);
    else if(IsBlessed())
	player->CreateMap(MAPPING_ALL, MyValue);
    else
	player->CreateMap(MAPPING_OBJECT, MyValue);
    
    return NULL;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
