// @ myth - server - sleep.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk
//
// Maintain a list of sleeping players

// @@ begin

#ifndef SLEEP_HEADER
#define SLEEP_HEADER

// @@ include

#include "common/list.h"
#include "common/string.h"
#include "common/command.h"
#include "server/id.h"
#include "server/global.h"

struct BINARY;
struct PLAYER;
struct PAGE;

// @@ struct sleep_link
//
// Used to contain info on each sleeping player.
// This class is only accesed by SLEEP

struct sleep_link
{
    id_type identity;
    STRING name;
    STRING passwd;
    level_type depth;
    void save(BINARY* binary);
    sleep_link(id_type id, BINARY* binary);
    sleep_link(STRING who, STRING how, level_type type, id_type id);
};

// @@ class SLEEP
//
// Operations on the list of sleeping players

class SLEEP : LIST
{
    // Return the current entry in the list
    inline sleep_link* current()
    { return (sleep_link*) LIST::current(); }
    // Remove and free current entry in the list
    void erase();
public:
    // Add a new player to the list
    void add(PLAYER* player);
    // Remove a player from the list
    void remove(id_type id);
    // Check if some sleeping player is using this name
    bool used(STRING who);
    // Get the id of a player, from his name and password
    id_type identify(STRING who, STRING how);
    // Find the level of sleeping player
    level_type where(id_type id);
    // Append the names of all slepping players to a page
    void page(PAGE* page);
    // Save this object
    void save(BINARY* binary);
    // Restore this object
    void load(BINARY* binary);
    // Allow initialisation
    SLEEP() { }
    // Clear and free memory for all entries on the list
    ~SLEEP();
};

// @@ end

#endif SLEEP_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
