// @ myth - server - specify.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "server/specify.h"
#include "common/error.h"
#include "server/rnd.h"
#include "server/monster.h"
#include "common/format.h"

// @@ const char* RockName(rock_type rock)

const char* RockName(rock_type rock)
{
    switch(rock)
    {
    case FLOOR:
	return "plain ground";
    case UNEVEN_FLOOR:
	return "rocky ground";
    case VERY_SOFT_ROCK:
	return "very soft rock";
    case SOFT_ROCK:
	return "soft rock";
    case ROCK:
	return "rock";
    case HARD_ROCK:
	return "hard rock";
    case VERY_HARD_ROCK:
	return "very hard rock";
    case  PERMANENT_ROCK:
	return "permanent rock";
    case ROCK_END:
    case ROCK_ERROR:
    default:
	bug.error("::RockName", format("Unknown rock type %d", rock));
	return "strange rock";
    }
}

// @@ int RockStrength(rock_type rock)

int RockStrength(rock_type rock)
{
    switch(rock)
    {
    case FLOOR:
	return 0;
    case UNEVEN_FLOOR:
	return 0;
    case VERY_SOFT_ROCK:
	return 1;
    case SOFT_ROCK:
	return 10;
    case ROCK:
	return 75;
    case HARD_ROCK:
	return 400;
    case VERY_HARD_ROCK:
	return 5000;
    case  PERMANENT_ROCK:
	// Really infinite, but that value is not supported on finite bit int's
	return 32000;
    case ROCK_END:
    case ROCK_ERROR:
    default:
	bug.error("::RockStrength", format("Unknown rock type %d", rock));
	return -1;
    }
}

// @@ SPECIFY

// @@@ bool SPECIFY::check(OBJECT*)

bool SPECIFY::check(OBJECT*)
{ return TRUE; }

// @@ SPECIFY_LETTER

// @@@ bool SPECIFY_LETTER::check(OBJECT* object)

bool SPECIFY_LETTER::check(OBJECT* object)
{
    return object->match(symbol, letter);
}

// @@@ SPECIFY_LETTER::SPECIFY_LETTER(symbol_type type, char c)

SPECIFY_LETTER::SPECIFY_LETTER(symbol_type type, char c)
{
    symbol = type;
    letter = c;
}

// @@ SPECIFY_WORN

// @@@ bool SPECIFY_WORN::check(OBJECT* object)

bool SPECIFY_WORN::check(OBJECT* object)
{
    return monster->pack.worn(object);
}

// @@ SPECIFY_PICK

// @@@ bool SPECIFY_PICK::check(OBJECT* object)

bool SPECIFY_PICK::check(OBJECT* object)
{
    if(!monster->IsPicking(object))
	return FALSE;
    else if(!object->CanRemove(monster))
	return FALSE;
    else if(!monster->LightLoad(extra + object->weight()))
	return FALSE;

    extra += object->weight();
    return TRUE;
}

// @@@ SPECIFY_PICK::SPECIFY_PICK(MONSTER* m)

SPECIFY_PICK::SPECIFY_PICK(MONSTER* m)
{
    monster = m;
    extra = 0L;
}

// @@ SPECIFY_WANT

// @@@ bool SPECIFY_WANT::check(OBJECT* object)

bool SPECIFY_WANT::check(OBJECT* object)
{
    return monster->IsEnemy(object) || monster->IsWanted(object);
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
