// @ myth - server - stair.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "server/stair.h"
#include "server/monster.h"
#include "common/binary.h"
#include "common/parse.h"
#include "common/error.h"
#include "server/global.h"
#include "common/format.h"

// @@ STAIR

// @@@ comp STAIR::compare(OBJECT* object)

comp STAIR::compare(OBJECT* object)
{
    comp c = OBJECT::compare(object);

    if(c == EQUAL)
        return ::compare(identity(), object->identity());
    else
        return c;
}

// @@@ bool STAIR::MoveUp(LEVEL*, MONSTER* monster)

bool STAIR::MoveUp(LEVEL*, MONSTER* monster)
{
    if(LevelUp == LEVEL_ERROR)
        monster->message("This stair does not lead up.");
    else if(!monster->CheckAbility(ABILITY_YENDOR))
        monster->message("That way is magically blocked.");
    else if(monster->strength() < 5)
        monster->message("You are to weak to climb up.");
    else
    {
        monster->NewLevel(LevelUp, WhereUp);
        monster->UseMovePoint(1000);
    }
    return TRUE;
}

// @@@ bool STAIR::MoveDown(LEVEL*, MONSTER* monster)

bool STAIR::MoveDown(LEVEL*, MONSTER* monster)
{
    if(LevelDown == LEVEL_ERROR)
        monster->message("This stair is not leading down.");
    else if(monster->CheckAbility(ABILITY_YENDOR))
        monster->message
    	("The Might of Yendor prevent you from descending further.");
    else
    {
        monster->NewLevel(LevelDown, WhereDown);
        monster->UseMovePoint(500);
    }
    return TRUE;
}

// @@@ void STAIR::save(BINARY* binary)

void STAIR::save(BINARY* binary)
{
    OBJECT::save(binary);
    binary->PutChar(MyLook);
    binary->PutChar(LevelUp);
    binary->PutChar(WhereUp);
    binary->PutChar(LevelDown);
    binary->PutChar(WhereDown);
}

// @@@ STAIR::STAIR(BINARY* binary)

STAIR::STAIR(BINARY* binary) : (binary)
{
    MyLook = binary->GetChar();
    LevelUp = binary->GetChar();
    WhereUp = binary->GetChar();
    LevelDown = binary->GetChar();
    WhereDown = binary->GetChar();
}

// @@@ STAIR::STAIR(PARSE* parse)

STAIR::STAIR(PARSE* parse) : (parse)
{
    MyLook = '+';
    LevelUp = LEVEL_ERROR;
    WhereUp = SYMBOL_ERROR;
    LevelDown = LEVEL_ERROR;
    WhereDown = SYMBOL_ERROR;
    
    char where = SYMBOL_ERROR;
    
    while(parse->good())
    {
        const char* string = parse->GetWord();
        parse->skip(INPUT_SPACE);
        
        /* BODY */
        if(strcmp(string, "UP") == EQUAL)
        {
	    WhereUp = where;
	    LevelUp = global.CaveToLevel(parse->GetString());
        }
        else if(strcmp(string, "DOWN") == EQUAL)
        {
	    WhereDown = where;
	    LevelDown = global.CaveToLevel(parse->GetString());
        }
        else if(strcmp(string, "WHERE") == EQUAL)
        {
	    parse->skip("\"");
	    where = parse->GetKey();
	    parse->skip("\"");
        }
        else if(strcmp(string, "LOOK") == EQUAL)
        {
	    parse->skip("\"");
	    MyLook = parse->GetKey();
	    parse->skip("\"");
        }
        else if(strcmp(string, "BREAK") == EQUAL)
	    break;
        else
	    parse->wrong("STAIR::STAIR", format("Unknown field: %s", string));

        parse->skip(INPUT_COMMENT);
    }
    parse->skip(INPUT_COMMENT);
}

// @@@ STAIR::STAIR(char l,

STAIR::STAIR(char l, level_type up, char u_pos, level_type down, char d_pos)
{
    MyLook = l;
    LevelUp = up;
    WhereUp = u_pos;
    LevelDown = down;
    WhereDown = d_pos;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
